from typing import TypeVar

TextType = TypeVar("TextType")
# AbstractNodeType = TypeVar("AbstractNodeType", bound=AbstractNode)
# AnalystType = TypeVar("AnalystType", bound=Analyst)
# AtomNodeType = TypeVar("AtomNodeType", bound=AtomNode)
# EdgeType = TypeVar("EdgeType", bound=Edge)
# MetadataType = TypeVar("MetadataType", bound=Metadata)
# ParticipantType = TypeVar("ParticipantType", bound=Participant)
# ReferenceType = TypeVar("ReferenceType", bound=Reference)
# ResourceType = TypeVar("ResourceType", bound=Resource)
# SchemeNodeType = TypeVar("SchemeNodeType", bound=SchemeNode)
# UserdataType = TypeVar("UserdataType", bound=Mapping)
